@@ -1,12 +1,20 @@
 #!/usr/bin/perl
+#
+# This is a basic, fairly fuctional Mason handler.pl.
+#
+# For something a little more involved, check out session_handler.pl
+
 package HTML::Mason;
 
 #
-# Sample Mason handler. At a minimum, set comp_root and data_dir.
+# Sample Mason handler.
 #
 use HTML::Mason;
 use strict;
 
+# Uncomment the next line if you plan to use the Mason previewer.
+#use HTML::Mason::Preview;
+
 # List of modules that you want to use from components (see Admin
 # manual for details)
 #{  package HTML::Mason::Commands;
@@ -25,8 +33,8 @@ my $ah = new HTML::Mason::ApacheHandler (interp=>$interp);
 # Resets ownership of all files created by Mason at startup. Change
 # these to match your server's 'User' and 'Group'.
 #
-#chown ( [getpwnam('nobody')]->[2], [getgrnam('nobody')]->[2],
-#        $interp->files_written );
+#chown (scalar(getpwnam "nobody"), scalar(getgrnam "nobody"),
+#       $interp->files_written);
 
 sub handler
 {
@@ -36,9 +44,11 @@ sub handler
     # components, activate the following to prevent Mason from
     # evaluating image files as components.
     #
-    #return -1 if $r->content_type !~ m|^text/|io;
+    #return -1 if $r->content_type && $r->content_type !~ m|^text/|io;
     
-    $ah->handle_request($r);
+    my $status = $ah->handle_request($r);
+    
+    return $status;
 }
 
 1;
