@@ -142,7 +142,7 @@ sub _initialize
     # Open system log file
     #
     if ($self->{system_log_events_hash}) {
-	$self->system_log_file($self->data_dir . "/etc/system.log") if !$self->system_log_file;
+	$self->{system_log_file} = $self->data_dir . "/etc/system.log" if !$self->system_log_file;
 	my $fh = new IO::File ">>".$self->system_log_file
 	    or die "Couldn't open system log file ".$self->{system_log_file}." for append";
 	$fh->autoflush(1);
@@ -489,6 +489,7 @@ sub find_comp_upwards
 
     my $comp;
     my $p = $startpath;
+    $p =~ s/\/$//;
     while (!($comp = $self->load("$p/$name")) && $p) {
 	my ($basename,$dirname) = fileparse($p);
 	$dirname =~ s/^\.//;    # certain versions leave ./ in $dirname
