@@ -14,13 +14,14 @@ use HTML::Mason::Tools qw(is_absolute_path read_file);
 use Carp;
 use strict;
 use vars qw($REQ $REQ_DEPTH %REQ_DEPTHS);
-my @_used = ($HTML::Mason::CODEREF_NAME,$::opt_P,$HTML::Mason::Commands::REQ);
+my @_used = ($HTML::Mason::CODEREF_NAME,$::opt_P,$HTML::Mason::Commands::m);
 
 my %fields =
     (aborted => undef,
      aborted_value => undef,
      count => 0,
      declined => undef,
+     error_code => undef,
      interp => undef,
      out_method => undef,
      out_mode => undef,
@@ -109,7 +110,10 @@ sub exec {
 		($self->{dhandler_arg} = $path) =~ s{^$parent/}{};
 	    }
 	}
-	die "could not find component for path '$path'\n" if !$comp;
+	unless ($comp) {
+	    $self->{error_code} = 'top_not_found';
+	    die "could not find component for initial path '$path'\n";
+	}
     } elsif (ref($comp) !~ /Component/) {
 	die "exec: first argument ($comp) must be an absolute component path or a component object";
     }
@@ -659,21 +663,23 @@ sub current_sink { return $_[0]->top_stack->{sink} }
 
 #
 # Return the absolute version of a component path. Handles . and ..
-# Empty string resolves to current component path.
+# Empty string resolves to current component path. Optional second
+# argument is directory path to resolve relative paths against.
 #
 sub process_comp_path
 {
-    my ($self,$compPath) = @_;
-    if ($compPath !~ /\S/) {
+    my ($self,$comp_path,$dir_path) = @_;
+    if ($comp_path !~ /\S/) {
 	return $self->current_comp->path;
     }
-    if ($compPath !~ m@^/@) {
-	die "relative component path ($compPath) used from component with no current directory" if !defined($self->current_comp->dir_path);
-	$compPath = $self->current_comp->dir_path . "/" . $compPath;
+    if ($comp_path !~ m@^/@) {
+	$dir_path = $self->current_comp->dir_path unless defined($dir_path);
+	die "relative component path ($comp_path) used from component with no current directory" unless $dir_path;
+	$comp_path = $dir_path . ($dir_path eq "/" ? "" : "/") . $comp_path;
     }
-    while ($compPath =~ s@/[^/]+/\.\.@@) {}
-    while ($compPath =~ s@/\./@/@) {}
-    return $compPath;    
+    while ($comp_path =~ s@/[^/]+/\.\.@@) {}
+    while ($comp_path =~ s@/\./@/@) {}
+    return $comp_path;    
 }
 
 1;
